/*****************************************************************************
*                            dat2vtk exchange program                        *
*                                Version 3.0-fusion3d                        *
*                                          2005.4.28                         *
*                              Copyright  Keita Ogawa                        *
*****************************************************************************/
#include "setup.h"

#define stp     (dbp)
#define glp     (dbp+tlp)
#define strdp   (dbp-um)
#define glrdp   (dbp+tlp)
#define header  "# vtk DataFile Version 2.0\n"
#define dformat "DATASET STRUCTURED_POINTS\n"
#define type    "ASCII\n"
#define pdata   ((int)((tlp/dx)+1)*jmp*kmp)
#define vectors "VECTORS vectors float\n"
#define unv     0.001

#define optErr    -1
#define optSuc     0
#define file_open  1
#define file_close 0

/*****************************************************************************
*                           Define each properties                           *
*****************************************************************************/
static double  F[3][imx+1][jmx+1][kmx+1];  /* Value of field (x)            */

static char fnm[FBUF+1],pnm[FBUF+1];
static FILE *fp;
static int   bf;                           /* File buffer number            */

void choice     (char *buf[]);
void getopt     (int  npt,char  *buf[],int argc,char *argv[]);
void extrac     (void);
void out_header (void);
void set_file   (int);
void out_vector (void);

int startup     (int argc,char *argv[]);

/*****************************************************************************
*                               Each programs                                *
*****************************************************************************/
int main(int argc,char *argv[])
{
  if(startup(argc,argv) == optErr) exit(1);
  
  extrac();
  
  set_file(file_open);
  out_header();
  out_vector();
  set_file(file_close);
  
  return optSuc;
}

void extrac(void)
{
  int i,j,k;
  
  fp = fopen(fnm,"r");
  
  if(fp == NULL){
    printf("Can't read a file ...\n");
    exit(1);
  }
  
  for(i=(int)(strdp/dx);i<=(int)(glrdp/dx);i++)
    for(j=0;j<jmx;j++)
      for(k=0;k<kmx;k++)
        fscanf(fp,"%lf %lf %lf\n",
               &F[0][i][j][k],&F[1][i][j][k],&F[2][i][j][k]);
  fclose(fp);
  
  printf("file reading is success ... %s\n",fnm);
}

void set_file(int flgs)
{
  if(flgs == file_open)
    {
      sprintf(pnm,"%c%cv%c%c%c%c.vtk",fnm[bf-11],fnm[bf-10],
	      fnm[bf-8],fnm[bf-7],fnm[bf-6],fnm[bf-5]);
      
      fp  = fopen(pnm,"w");
      
      if(fp == NULL) exit(1);
      printf("making is success       ... %s\n",pnm); 
    }
  
  if(flgs == file_close){
    fclose(fp);
    printf("closing is success      ... %s\n\n",pnm);
  }
}

void out_header(void)
{
  /* writing header */
  fprintf(fp,header);
  
  /* writing title  */
  fprintf(fp,"%s\n",pnm);
  
  /* writing ASCII or BINARY */
  fprintf(fp,type);
  fprintf(fp,"\n");
  
  /* writing data format */
  fprintf(fp,dformat);
  
  /* writing Dimensions */
  fprintf(fp,"DIMENSIONS %d %d %d\n",(int)(((tlp)/dx)+1),jmx+1,kmx+1);
  
  /* writing Origin */
  fprintf(fp,"ORIGIN %f %f %f\n",stp/twp,0.0,0.0);
  
  /* writing Spacing */
  fprintf(fp,"SPACING %f %f %f\n",dx/twp,dy/twp,dz/twp);
  fprintf(fp,"\n");
  
  /* writing Point data */
  fprintf(fp,"POINT_DATA %d\n",pdata);
  
  /* writing Vectors */
  fprintf(fp,vectors);
  fprintf(fp,"\n");
}

void out_vector(void)
{
  int i,j,k,m=0;
  
  for(k=0;k<=kmx;k++){
    for(j=0;j<=jmx;j++){
      for(i=(int)(stp/dx);i<(int)(glp/dx);i++){
        if(fabs(F[0][i][j][k]) < unv) F[0][i][j][k] = 0.0;
        if(fabs(F[1][i][j][k]) < unv) F[1][i][j][k] = 0.0;
        if(fabs(F[2][i][j][k]) < unv) F[2][i][j][k] = 0.0;
        
	if(i != (int)(glp/dx))
	  fprintf(fp,"%f %f %f\t",F[0][i][j][k],F[1][i][j][k],F[2][i][j][k]);
	else
	  fprintf(fp,"%f %f %f" , F[0][i][j][k],F[1][i][j][k],F[2][i][j][k]);
        m++;
      }
      fprintf(fp,"\t");
    }
    fprintf(fp,"\n");
  }
  printf("writing counter         ... %d\n",m);
}

/************************** shell option command *****************************/
void getopt(int opt,char *buf[],int argc,char *argv[])
{
  int i = 0;
  
  do{
    if(i > FBUF)
      {
        printf("Argv chars are too big ...\n");
        exit(1);
      }
    
    buf[i] = (argv[opt]+i);
    i++;
  }while(*(argv[opt]+i) != '\0');
  
  *(buf[0]+i) = '\0';
}

int startup(int argc,char *argv[])
{
  char *buf[FBUF+1];
  int i = 0;
  
  if(argc < 2)
    printf("No parameter ...\n");
  else if(argc >= 3)
    printf("A lot of parameter ...\n");
  else
    {
      /* get and arrange arguments */
      getopt(1,buf,argc,argv);
      
      while(*(buf[0]+i) != '\0')
        {
          fnm[i] = *buf[i];
          i++;
        }
      bf = i;
      return optSuc;
    }
  return optErr;
}
/*****************************************************************************
*                            dat2vtk exchange program                        *
*                                Version 3.0-fusion3d                        *
*                                          2005.4.28                         *
*                              Copyright  Keita Ogawa                        *
*****************************************************************************/
